#!/usr/bin/env python3
"""
Classes for Getting and Calculating Data.
General-purpose solar irradiance and brightness calculator.
"""
import logging
from datetime import datetime, timedelta
import requests
from requests.exceptions import HTTPError


class Basedata:
    """
    BaseData class: Collects the relevant Data neccessary
    to calculate Illumination and Irradiance.
    """
    def __init__(self,
                 name=None,
                 api_key=None,
                 api_base=None,
                 api_loc=None,
                 city=None,
                 country=None,
                 requested_day=None,
                 requested_hour=None
                 ):
        logging.info("Initializing BaseData class.")
        if name is None:
            name = "basedata"
        self.name = name
        if api_key is None:
            self.api_key = None
        elif isinstance(api_key, str):
            self.api_key = api_key
        if api_base is None:
            self.api_base: str = 'https://api.openweathermap.org/'
        elif isinstance(api_base, str): 
            self.api_base = api_base
        if city is None:
            self.city = None
        elif isinstance(city, str):
            self.city = city
        if country is None:
            self.country = None
        elif isinstance(country, str):
            self.country = country
        if api_loc is None:
            self.api_loc: str = 'geo/1.0/direct?'
        elif isinstance(api_loc, str):
            self.api_loc = api_loc
        if requested_day is not None:
            self.requested_day = requested_day
        else:
            self._requested_day = None
        if requested_hour is not None:
            self.requested_hour = requested_hour
        else:
            self._requested_hour = None
     
     
    @property
    def sunrise_datetime(self):
        return self._sunrise_datetime
    
    @sunrise_datetime.setter
    def sunrise_datetime(self, value):
        logging.debug(f'set _sunrise_datetime to {value}')
        self._sunrise_datetime = value
    
    @property
    def sunset_datetime(self):
        return self._sunset_datetime
    
    @sunset_datetime.setter
    def sunset_datetime(self, value):
        logging.debug(f'set _sunset_datetime to {value}')
        self._sunset_datetime = value
    
    @property
    def api_key(self):
        return self._api_key
    
    @api_key.setter
    def api_key(self, value):
        logging.info(f"Setting api_key: {value}")
        self._api_key = str(value)
        
    @property
    def city(self):
        return self._city
    
    @city.setter
    def city(self, value):
        logging.info(f"Setting city: {value}")
        self._city = str(value)
    
    @property
    def country(self):
        return self._country
    
    @country.setter
    def country(self, value):
        logging.info(f"Setting country: {value}")
        self._country = str(value)
    
    @property
    def requested_day(self):
        return self._requested_day
    
    @requested_day.setter
    def requested_day(self, value):
        logging.info(f"Setting requested_day: {value}")
        self._requested_day = str(value)

    @property
    def requested_hour(self):
        return self._requested_hour
    
    @requested_hour.setter
    def requested_hour(self, value):
        logging.info(f"Setting requested_hour: {value}")
        self._requested_hour = str(value)

    @property
    def current_date(self):
        """
        Returns current date as string (YYY-MM-DD),
        If a custom value is provided (self.requested_day)
        it will be returned instead.
        """
        if self.requested_day is not None and isinstance(self.requested_day, str):
            _current_date = self.requested_day
            logging.debug(f"{self.requested_day} is a valid date")
        else:
            logging.debug(f"{self.requested_day} is a not valid date")
            now = datetime.now()
            _current_date = now.strftime("%Y-%m-%d")
        logging.info(f"set current date to {_current_date}")
        return _current_date

    @property
    def day_of_the_year(self):
        """
         @brief Returns the day of the year. It is defined as the number of days since January 1 1970 00 : 00 : 00.
         @return a tuple of the form ( year month day )
        """
        _date_object = datetime.strptime(self.current_date, "%Y-%m-%d")
        _day_of_the_year = _date_object.timetuple().tm_yday
        logging.info(f"calculated day of the year: {_day_of_the_year}")
        return _day_of_the_year

    @property
    def current_time(self):
        """
        Returns current hour of day as string (e.g. 12),
        If a custom value is provided (self.requested_hour)
        it will be returned instead.
        """
        #print("current_time accessed")
        if self.requested_hour is not None and isinstance(self.requested_hour, str):
            _current_time = self.requested_hour
            logging.debug(f"{self.requested_hour} is a valid hour")
        else:
            now = datetime.now()
            logging.debug(f"{self.requested_hour} is a not valid hour")
            logging.debug(f'setting current_time to actual time')
            _current_time = now.strftime("%-H")
        logging.info(f"set current time to {_current_time}")
        return _current_time

    @property
    def latitude(self):
        """
        Calls OpenWeatherMap API to get latitude of the city and
        returns it as "latitude".
        """
        logging.debug('getting latitude')
        _parameters = {
            "q": f"{self.city},{self.country}",
            "limit": 1
        }
        logging.debug(f"parameters: {_parameters}")
        _api_suburl = self.api_loc
        _api_response = self.requester(_api_suburl, _parameters)
        # _api_response = _api_response_raw
        _latitude = _api_response[0]['lat']
        logging.info(f"latitude: {_latitude}")
        return float(_latitude)

    @property
    def longitude(self):
        """
        Calls OpenWeatherMap API to get longitude of the city and
        returns it as "longitude".
        """
        logging.debug('getting longitude')
        _parameters = {
            "q": f"{self.city},{self.country}",
            "limit": 1
        }
        logging.debug(f"parameters: {_parameters}")
        _api_suburl = str(self.api_loc)
        _api_response = self.requester(_api_suburl, _parameters)
        # _api_response = _api_response_raw.json()
        _longitude = _api_response[0]['lon']
        logging.info(f"longitude: {_longitude}")
        return float(_longitude)

    def requester(self, _api_suburl: str, parameters: dict):
        """
        Returns the response of the API request.
        Takes the specific sub-url and parameters as input.
        """
        logging.debug('using requester')
        _api_url: str = f"{self.api_base}{_api_suburl}"
        parameters["appid"] = self.api_key
        _parameters = parameters
        logging.debug(f"parameters: {_parameters}")
        try:
            _response = requests.get(_api_url, params=_parameters, timeout=10)
            _response.raise_for_status()
            # _response = _response_raw.json()
            logging.debug(f"response: {_response.json()}")
            return _response.json()
        except HTTPError as err:
            raise SystemExit(err) from err
       
    @property
    def tester(self):
        """
        test function.
        """
        _parameters = {
            "q": f"{self.city},{self.country}",
            "limit": 1
            }
        _api_suburl = self.api_loc
        _api_response = self.requester(_api_suburl, _parameters)
        # _api_response = _api_response_raw
        testdata = (_api_response)[0]['name']
        return testdata


class Weather(Basedata):
    """
    Weather class:
    Using the OpenWeatherMap API, get current weather
    infomation for given time and location.
    """
    def __init__(self,
                 name=None,
                 api_key=None,
                 api_base=None,
                 api_loc=None,
                 api_weather=None,
                 city=None,
                 country=None,
                 ):
        if name is None:
            name = "weather"
        self.name = name
        super().__init__(self)
        # self.api_key = api_key
        # self.api_base = api_base
        # self.city = city
        # self.country = country
        # self.api_loc = api_loc
           
        if api_weather is None:
            self.api_weather: str = '/data/2.5/weather?'
        elif isinstance(api_weather, str):
            self.api_weather = api_weather
        self.sunrise_unix = None
        self.sunset_unix = None
        self._cloud_coverage = None
        self._sunrise = None
        self._sunset = None
        self._sunrise_datetime = None
        self._sunset_datetime = None
        self._timezone = None
        
    @property
    def timezone(self):
        if self._timezone is None:
            self.get_weather
        return self._timezone
        
    @timezone.setter
    def timezone(self, value: int):
        _timezone = value / (60**2)
        self._timezone = _timezone
    
    @property
    def sunset(self):
        self.sunset = datetime.strftime(self.sunset_datetime, "%H:%M:%S")
        return self._sunset
    
    @sunset.setter
    def sunset(self, value):
        self._sunset = value
    
    @property
    def sunrise(self):
        self.sunrise = datetime.strftime(self.sunrise_datetime, "%H:%M:%S")
        return self._sunrise
          
    @sunrise.setter
    def sunrise(self, value):
        self._sunrise = value


    @property
    def sunrise_datetime(self):
        if self._sunrise_datetime is None:
            self.suntimes()
        return self._sunrise_datetime
    
    @sunrise_datetime.setter
    def sunrise_datetime(self, value):
        if value is not None and isinstance(value, datetime):
            _sunrise_datetime = value
            logging.info(f"Setting sunrise datetime: {_sunrise_datetime}")
        else:
            raise TypeError(f"{value} must be Type datetime")
        self._sunrise_datetime = _sunrise_datetime

    @property
    def sunset_datetime(self):
        if self._sunset_datetime is None:
            self.suntimes()
        return self._sunset_datetime
    
    @sunset_datetime.setter
    def sunset_datetime(self, value):
        if value is not None and isinstance(value, datetime):
            _sunset_datetime = value
            logging.info(f"Setting sunset datetime: {_sunset_datetime}")
        else:
            raise TypeError(f"{value} must be Type datetime")
        self._sunset_datetime = _sunset_datetime

    def get_weather(self):
        """
        Calls OpenWeatherMap API to get cloud coverage of the city and
        returns it as "cloud_coverage".
        """
        logging.debug('getting weather data')
        _parameters = {
            "lat": self.latitude,
            "lon": self.longitude,
            }
        _api_suburl = self.api_weather
        _api_response = self.requester(_api_suburl, _parameters)
        # _api_response = _api_response_raw.json()
        _sunrise = _api_response['sys']['sunrise']
        _sunset = _api_response['sys']['sunset']
        _timezone= _api_response['timezone']
        logging.info(f"sunrise: {_sunrise}")
        logging.info(f"sunset: {_sunset}")
        # _time_shift = _api_response['timezone']
        self.timezone = int(_timezone)
        self.sunrise_unix = int(_sunrise)
        self.sunset_unix = int(_sunset)
        if self.requested_day is not None:
            _cloud_coverage = 0
        else:
            _cloud_coverage = _api_response['clouds']['all']
        self._cloud_coverage = float(_cloud_coverage)
        logging.info(f"cloud coverage: {_cloud_coverage}")
        
          
    @property
    def cloud_coverage(self):
        logging.debug('getting cloud coverage')
        if self._cloud_coverage is None:
            self.get_weather()
        if isinstance(self._cloud_coverage, float):
            _cloud_coverage = self._cloud_coverage
            return _cloud_coverage
        else:
            raise TypeError("Cloud coverage must be a float.")

    def suntimes(self):
        if self.sunrise_unix or self.sunset_unix is None:
            self.get_weather()
        if isinstance(self.sunrise_unix, int) and isinstance(self.sunset_unix, int):
            _rise = datetime.fromtimestamp(self.sunrise_unix)
            _set = datetime.fromtimestamp(self.sunset_unix)
        else:
            raise TypeError("Sunrise and Sunset must be unix timestamps.")
        
        self.sunrise_datetime = _rise
        self.sunset_datetime = _set
        
        
    @property
    def sun_up(self):
        _rise: datetime = self.sunrise_datetime
        _set: datetime = self.sunset_datetime
        _date_actual = datetime.now()
        _date_actual_str = _date_actual.strftime("%Y-%m-%d")
        logging.debug(f"current date: {_date_actual_str}")
        _hour = float(self.current_time)
        _date = datetime.strptime(_date_actual_str, "%Y-%m-%d")
        _now = _date + timedelta(hours=_hour)
        logging.info(f"sunrise timestamp is {_rise}")
        logging.info(f"sunset timestamp is {_set}")
        logging.info(f"current time is {_now}")
        if _rise < _now < _set:
            _sun_up = True
        else:
            _sun_up = False
        logging.info(f"sun up: {_sun_up}")
        return _sun_up


#!/usr/bin/env python3
"""
__main__ module for running from cli
"""

import logging
import argparse
import sys
from sys import stderr
from .data import light_data
from .defaults import CITY_DEFAULT, COUNTRY_DEFAULT, API_KEY_DEFAULT

if __name__ == "__main__":
    argParser = argparse.ArgumentParser(
        description='General-purpose solar brightness calculator.'
        )
    argParser.add_argument(
        "-t", "--time",
        default=None,
        help="Time of the day in hours (0-24, e.g. 4 or 12). \
            Defaults to current time if not specified."
        )
    argParser.add_argument(
        "-d",
        "--date",
        default=None,
        help="Date as YYYY-MM-DD (e. g. 2023-06-01). \
            Defaults to current date if not specified."
        )
    argParser.add_argument(
        "-c", "--city",
        default=CITY_DEFAULT,
        help="City name (e. g. Berlin). \
            Defaults to {CITY_DEFAULT} if not specified."
        )
    argParser.add_argument(
        "-C",
        "--Country",
        default=COUNTRY_DEFAULT,
        help="Country name (e. g. Germany). \
            Defaults to {COUNTRY_DEFAULT} if not specified."
        )
    argParser.add_argument(
        "-a",
        "--api",
        default=API_KEY_DEFAULT,
        help="OpeWeather API-Key (required). \
            Alternatively, a default API key can be defined in main.py"
        )
    argParser.add_argument(
        "-o",
        "--output",
        default=['illuminance'],
        help="Define the parameters to be returned. \
            Defaults to 'Illuminance' if not specified.\
                    To see available values try illumipy\
                        [-o|--output] help",
        nargs='+',
        )
    argParser.add_argument(
        '-D', '--debug',
        help="Print lots of debugging statements",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.WARNING,
    )
    argParser.add_argument(
        '-v',
        '--verbose',
        help="Be verbose",
        action="store_const",
        dest="loglevel",
        const=logging.INFO,
    )
    argParser.add_argument(
        "-A",
        "--all",
        help="Print out entire dataset. Default is just the Illuminance in Lux",
        action="store_true",
        default=False,
    )
    argParser.add_argument(
        "-s",
        "--short",
        help="Print only values, no description",
        action="store_true",
        default=False,
        )



    args = argParser.parse_args()

    LOG_FORMAT = '[%(levelname)s]:  \t%(module)s.%(funcName)s() \
        (Ln. %(lineno)d):\n\t\t\t>>> %(message)s\n'
    logging.basicConfig(level=args.loglevel, format=LOG_FORMAT)

    time_arg = args.time
    date_arg = args.date
    city_arg = args.city
    country_arg = args.Country
    api_arg = args.api
    print_all = args.all
    output_arg = args.output
    short_arg = args.short
    
    if output_arg == ['help']:
        print("Available values for output:\n\
                \t['illuminance']: Outside Brightness in Lux\n\
                \t['time']: Time used as %-H (e.g. 4 or 12)\n\
                \t['date']: Date used as YYYY-MM-DD\n\
                \t['city']: City Used\n\
                \t['country']: Country Used\n\
                \t['cloud_coverage']: Cloud coverage in %\n\
                \t['et_illuminance']: Extraterrestrial Illuminance in Lux\n\
                \t['direct_illuminance']: Direct Illuminance in Lux\n\
                \t['horizontal_illuminance']: Horizontal Illuminance in Lux\n\
                \t['horizontal_sky_illuminance']: Horizontal Sky Illuminance in Lux\n\
                \t['sunrise']: Time of Sunrise as hh:mm\n\
                \t['sunset']: Time of sunset as hh:mm\n\
                \t['sun_altitude']: Sund altitude at [Time] in degrees\n\
                \t['day']: True if there is daylight at [Time]\n\
                \t['clear_sky_index'] = Aproximation of Clear Sky Index based on cloud coverage\n\
                \t['cs_irradiance'] = Estimated Clear Sky Irradiance in W/m^2 based on solar altitude.\n\
                \t['irradiance'] = Estimated current Irradiance in W/m^2 based on solar altitude and cloud coverage.")
        exit()
        
    args_debug = f"time={time_arg}, date={date_arg}, city={city_arg},\
        country={country_arg}, api_key={api_arg}"
    logging.info('Calling main function in main.py')
    logging.debug('Using these args: %s', args_debug)

    try:
        brightness = light_data(time=time_arg,
                        date=date_arg,
                        city=city_arg,
                        country=country_arg,
                        api_key=api_arg)
        if print_all is True:
            dataset = brightness.keys()
        else:
            dataset = output_arg
        for item in dataset:
            if short_arg is True:
                print(brightness[item])
            else:
                print(f"{item}: {brightness[item]}")
    except KeyboardInterrupt:
        stderr.write("Interrupted by User, exiting...\n")
        sys.exit(1)



#!/usr/bin/env python3

"""
General-purpose solar irradiance and brightness calculator.
"""
import logging
from illumipy.classes.illumination import Illumination
from illumipy.defaults import API_KEY_DEFAULT, CITY_DEFAULT, COUNTRY_DEFAULT


def light_data(time=None, date=None, city=None, country=None, api_key=None, cloud_coverage=None):
    """ Main function:
    Creates Illuminance object to calculate Relevant data and
    returns a dictionary object with the following information:
        ['illuminance']: Outside Brightness in Lux
        ['time']: Time used as %-H (e.g. 4 or 12)
        ['date']: Date used as YYYY-MM-DD
        ['city']: City Used
        ['country']: Country Used
        ['cloud_coverage']: Cloud coverage in %
        ['et_illuminance']: Extraterrestrial Illuminance in Lux
        ['direct_illuminance']: Direct Illuminance in Lux
        ['horizontal_illuminance']: Horizontal Illuminance in Lux
        ['horizontal_sky_illuminance']: Horizontal Sky Illuminance in Lux
        ['sunrise']: Time of Sunrise as hh:mm
        ['sunset']: Time of sunset as hh:mm
        ['sun_altitude']: Sund altitude at [Time] in degrees.
        ['day']: True if there is daylight at [Time].
        ['clear_sky_index'] = Aproximation of Clear Sky Index based on cloud coverage
        ['cs_irradiance'] = Estimated Clear Sky Irradiance in W/m^2 based on solar altitude.
        ['irradiance'] = Estimated current Irradiance in W/m^2 based on solar altitude and cloud coverage.
    Takes the following arguments:
        time: str=[0-24], date: str=[YYYY-MM-DD], city: str=['City'],
        country: str=['Country'], api_key: str=['api key']
    If no arguments are provided, defaults to values defined in main.py.
    """
    _time = time
    logging.debug('time is %s', _time)
    _date = date
    logging.debug('date is %s', _date)
    if city is not None:
        _city = city
    else:
        _city = CITY_DEFAULT
    logging.debug('city is %s', _city)
    if country is not None:
        _country = country
    else:
        _country = COUNTRY_DEFAULT
    logging.debug('country is %s', _country)
    if api_key is not None:
        _api_key = api_key
    else:
        _api_key = API_KEY_DEFAULT
    logging.debug('api_key is %s', _api_key)
    if cloud_coverage is not None:
        _cloud_coverage = cloud_coverage
    else:
        _cloud_coverage=None
    # illumination = initialiser(requested_hour=12)
    logging.debug('initialising illumination object')
    illumination = Illumination(
        requested_hour=_time,
        requested_day=_date,
        city=_city,
        country=_country,
        api_key=_api_key,
        cloud_coverage=_cloud_coverage)

    illumination_data = {}
    illumination_data['illuminance'] = illumination.daylight_illuminance
    illumination_data['time'] = illumination.current_time
    illumination_data['date'] = illumination.current_date
    illumination_data['city'] = illumination.city
    illumination_data['country'] = illumination.country
    illumination_data['cloud_coverage'] = illumination.cloud_coverage
    illumination_data['et_illuminance'] = illumination.et_illuminance
    illumination_data['direct_illuminance'] = illumination.direct_illuminance
    illumination_data[
        'horizontal_illuminance'
        ] = illumination.horizontal_illuminance
    illumination_data[
        'horizontal_sky_illuminance'
        ] = illumination.horizontal_sky_illuminance
    illumination_data['sunrise'] = illumination.sunrise
    illumination_data['sunset'] = illumination.sunset
    illumination_data['sun_altitude'] = illumination.altitude
    illumination_data['day'] = illumination.day
    illumination_data['clear_sky_index'] = illumination.clear_sky
    illumination_data['cs_irradiance'] = illumination.irradiance_clear
    illumination_data['irradiance'] = illumination.irradiance_cloud
   illumination_data['air_mass'] = illumination.air_mass
    return illumination_data
